/********************************************************************
 * Project:    STR9-comStick GNU (ADC + UART)
 * File:       main.c
 *
 * System:     ARM9TDMI 32 Bit (STR912FW44X)
 * Compiler:   GCC 4.0.3
 *
 * Date:       2006-12-20
 * Author:     Applications@Hitex.de
 *
 * Rights:     Hitex Development Tools GmbH
 *             Greschbachstr. 12
 *             D-76229 Karlsruhe
 ********************************************************************
 * Description:
 *
 * This file is part of the GNU Example chain for STR9-comStick
 * The code is bassed on usage of the STmicro library functions
 * This is a small implementation of UART1 feature sending AD converted values
 * The application runs in ARM mode with high optimization level.
 *
 ********************************************************************
 * History:
 *
 *    Revision 1.0    2006/12/20      Gn
 *    Initial revision
 ********************************************************************
 * This is a preliminary version.
 *
 * WARRANTY:  HITEX warrants that the media on which the SOFTWARE is
 * furnished is free from defects in materials and workmanship under
 * normal use and service for a period of ninety (90) days. HITEX entire
 * liability and your exclusive remedy shall be the replacement of the
 * SOFTWARE if the media is defective. This Warranty is void if failure
 * of the media resulted from unauthorized modification, accident, abuse,
 * or misapplication.
 *
 * DISCLAIMER:  OTHER THAN THE ABOVE WARRANTY, THE SOFTWARE IS FURNISHED
 * "AS IS" WITHOUT WARRANTY OF ANY KIND. HITEX DISCLAIMS ALL OTHER WARRANTIES,
 * EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * NEITHER HITEX NOR ITS AFFILIATES SHALL BE LIABLE FOR ANY DAMAGES ARISING
 * OUT OF THE USE OF OR INABILITY TO USE THE SOFTWARE, INCLUDING DAMAGES FOR
 * LOSS OF PROFITS, BUSINESS INTERRUPTION, OR ANY SPECIAL, INCIDENTAL, INDIRECT
 * OR CONSEQUENTIAL DAMAGES EVEN IF HITEX HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGES.
 ********************************************************************/

#include "defines.h"

#define global extern   /* to declare external variables and functions      */
#include "91x_lib.h"

#include "main.h"

#include <stdio.h>
#include <string.h>

char UARTDebug_TxBuffer[100];
char UART_TxBuffer[100];
void UARTDebugConfig(void);
void UARTDebugSend(char* Buffer, u32 count);
UART_InitTypeDef      UART_InitStructure;     /*  Init structure for the UART peripherie  */
GPIO_InitTypeDef      GPIO_InitStructure;     /*  Init structure for the GPIO peripherie  */

s8 UARTDebug_RxBuffer[100];                   /*  Transmit buffer  */
s8 UARTDebug_RxCounter = 0;                   /*  number of bytes to transmit  */
   
static void Delay(u32 nCount);

int main (void)
{
   UARTDebugConfig();
   
   sprintf(UART_TxBuffer, "\n\r****UART Send Test Frame****\n\r");

   /* endless loop */
   while (1)
   { 
      UARTDebugSend(UART_TxBuffer, 35);
      Delay(0xFFFFF);
   }
}


/*******************************************************************************
* Function Name  : Delay
* Description    : Inserts a delay time.
* Input          : nCount: specifies the delay time length.
*******************************************************************************/
static void Delay(u32 nCount)
{
   u32 j = 0;

   for(j = nCount; j != 0; j--);
}
/************************************** EOF *********************************/

/***********************************************************************************************************************
 *  UARTDebug_SCUConfig
 **********************************************************************************************************************/
/*! \brief    Method configures the system control unit for the usage of the UART peripherie. GPIO3 is the port
              of Rx/Tx pin.
 *  \pre      -
 *  \context  -
 *  \note     -
 **********************************************************************************************************************/
void UARTDebug_SCUConfig(void)
{
  SCU_APBPeriphClockConfig(__GPIO3, ENABLE);  /*  Enable the GPIO3 Clock  */
  SCU_AHBPeriphReset(__GPIO3,DISABLE);
  
  /* Enable the UART1 Clock */
  /* TODO: Remove UART1 Test Output */
  SCU_APBPeriphClockConfig(__UART1, ENABLE);
}

/***********************************************************************************************************************
 *  UARTDebug_GPIOConfig
 **********************************************************************************************************************/
/*! \brief    Method configures the GPIO peripherie for the usage of the UART.
              Rx pin is P3.2; Tx pin is P3.3
 *  \pre      -
 *  \context  -
 *  \note     -
 **********************************************************************************************************************/
void UARTDebug_GPIOConfig(void)
{
  /******************************************************************************************
  *  Configuration of UART1 TX/RX Pins (GPIO3)
  ******************************************************************************************/  
  /* TODO: Remove UART1 Test Output (GPIO3) */
  /* configure UART1_Rx pin GPIO3.2*/
  GPIO_InitStructure.GPIO_Direction = GPIO_PinInput;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
  GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull ;
  GPIO_InitStructure.GPIO_IPConnected = GPIO_IPConnected_Enable;
  GPIO_InitStructure.GPIO_Alternate = GPIO_InputAlt1  ;
  GPIO_Init (GPIO3, &GPIO_InitStructure);
  /*Gonfigure UART1_Tx pin GPIO3.3*/
  GPIO_InitStructure.GPIO_Direction = GPIO_PinInput;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
  GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull ;
  GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt2  ;
  GPIO_Init (GPIO3, &GPIO_InitStructure); 
}


/* TODO: Remove UART1 configuration ***********************************************************************/
void UARTDebugConfig(void)
{
  UARTDebug_SCUConfig();
  UARTDebug_GPIOConfig();
  
  
  UART_InitStructure.UART_WordLength = UART_WordLength_8D;
  UART_InitStructure.UART_StopBits = UART_StopBits_1;
  UART_InitStructure.UART_Parity = UART_Parity_No ;
  UART_InitStructure.UART_BaudRate = 115200;
  UART_InitStructure.UART_HardwareFlowControl = UART_HardwareFlowControl_None;
  UART_InitStructure.UART_Mode = UART_Mode_Tx_Rx;
  UART_InitStructure.UART_FIFO = UART_FIFO_Enable;//UART_FIFO_Enable;
  UART_InitStructure.UART_TxFIFOLevel = UART_FIFOLevel_1_2; /* FIFO size 16 bytes, FIFO level 8 bytes */
  UART_InitStructure.UART_RxFIFOLevel = UART_FIFOLevel_1_2; /* FIFO size 16 bytes, FIFO level 8 bytes */

  UART_DeInit(UART1);
  UART_Init(UART1, &UART_InitStructure);

  /* Enable the UART1 */
  UART_Cmd(UART1, ENABLE);
  
  /* Send Welcome message */
  sprintf(UARTDebug_TxBuffer, "\n\r****UARTDebug Connection****\n\r");
  UARTDebugSend(UARTDebug_TxBuffer, 32); 
}
/*********************************************************************************************************************/
void UARTDebugSend(char* Buffer, u32 count)
{
   u32 i;
   for(i=0; i < count; i++)
   {
      UART_SendData(UART1, Buffer[i]);
      while(UART_GetFlagStatus(UART1, UART_FLAG_TxFIFOFull) != RESET);
   }
}
